"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _path = require("path");

var _helperPluginUtils = require("@babel/helper-plugin-utils");

var _core = require("@babel/core");

var _fs = _interopRequireDefault(require("fs"));

var _lodash = _interopRequireDefault(require("lodash.template"));

var _lodash2 = _interopRequireDefault(require("lodash.some"));

var _findUp = _interopRequireDefault(require("find-up"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance"); }

function _iterableToArrayLimit(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

const REQUIRE = 'require';
const DEFAULT_CONFIG_NAMES = ['alias.config.js', 'app.config.js', 'webpack.config.js', 'webpack.config.babel.js'];

function fileExists(path) {
  try {
    return !_fs.default.accessSync(path, _fs.default.F_OK);
  } catch (e) {
    return false;
  }
}

function getConfigPath(filename, configPaths, findConfig) {
  let conf = null; // Try all config paths and return for the first found one

  (0, _lodash2.default)(configPaths, configPath => {
    if (!configPath) return false; // Compile config using environment variables

    const compiledConfigPath = (0, _lodash.default)(configPath)(process.env);
    let resolvedConfigPath;

    if (!findConfig) {
      // Get webpack config
      resolvedConfigPath = (0, _path.resolve)(process.cwd(), compiledConfigPath);
    } else {
      resolvedConfigPath = _findUp.default.sync(compiledConfigPath, {
        cwd: (0, _path.dirname)(filename),
        type: 'file'
      });
    }

    if (resolvedConfigPath && fileExists(resolvedConfigPath)) {
      conf = resolvedConfigPath;
    }

    return conf;
  });
  return conf;
}

const cached = {};

var _default = (0, _helperPluginUtils.declare)(api => {
  api.assertVersion(7);
  return {
    name: 'babel-plugin-alias-config',
    visitor: {
      CallExpression(path, _ref) {
        let filename = _ref.file.opts.filename,
            _ref$opts = _ref.opts;
        _ref$opts = _ref$opts === void 0 ? {} : _ref$opts;
        let configPath = _ref$opts.config,
            _ref$opts$findConfig = _ref$opts.findConfig,
            findConfig = _ref$opts$findConfig === void 0 ? false : _ref$opts$findConfig,
            _ref$opts$noOutputExt = _ref$opts.noOutputExtension,
            noOutputExtension = _ref$opts$noOutputExt === void 0 ? false : _ref$opts$noOutputExt;
        const nodeArguments = path.node.arguments; // If not a require statement do nothing

        if (!_core.types.isIdentifier(path.node.callee, {
          name: REQUIRE
        })) {
          return;
        } // Make sure required value is a string


        if (nodeArguments.length === 0 || !_core.types.isStringLiteral(nodeArguments[0])) {
          return;
        }

        const configPaths = configPath ? [configPath].concat(DEFAULT_CONFIG_NAMES) : DEFAULT_CONFIG_NAMES; // Get webpack config

        const confPath = getConfigPath(filename, configPaths, findConfig); // If the config comes back as null, we didn't find it, so throw an exception.

        if (!confPath) {
          throw new Error(`Cannot find any of these configuration files: ${configPaths.join(', ')}`);
        } // Because of babel-register, babel is actually run on webpack config files using themselves
        // as config, leading to odd errors


        if (filename === (0, _path.resolve)(confPath)) return;
        let aliasConf;
        let extensionsConf;
        let cwd;
        let aliases;
        let cache = cached[confPath];

        if (cache) {
          if (!cache.conf) return;
          if (cache.error) throw cache.error; // eslint-disable-next-line

          aliasConf = cache.aliasConf; // eslint-disable-next-line

          extensionsConf = cache.extensionsConf; // eslint-disable-next-line

          cwd = cache.cwd; // eslint-disable-next-line

          aliases = cache.aliases;
        } else {
          // Require the config
          let conf = require(confPath); // if the object is empty, we might be in a dependency of the config - bail without warning


          if (!Object.keys(conf).length) {
            return;
          }

          cwd = (0, _path.dirname)(confPath);
          cache = {
            conf,
            cwd
          };
          cached[confPath] = cache; // In the case the webpack config is an es6 config, we need to get the default
          // eslint-disable-next-line

          if (conf && conf.__esModule && conf.default) {
            conf = conf.default;
          } // exit if there's no alias config and the config is not an array


          if (!conf.alias && !(conf.resolve && conf.resolve.alias) && !Array.isArray(conf)) {
            cache.error = new Error("The resolved config file doesn't contain a resolve configuration");
            throw cache.error;
          } // Get the webpack alias config


          if (Array.isArray(conf)) {
            // the exported webpack config is an array ...
            // (i.e., the project is using webpack's multicompile feature) ...
            // reduce the configs to a single alias object
            aliasConf = conf.reduce((prev, curr) => {
              const next = Object.assign({}, prev);
              const alias = curr.alias || curr.resolve && curr.resolve.alias;

              if (alias) {
                Object.assign(next, alias);
              }

              return next;
            }, {}); // if the object is empty, bail

            if (!Object.keys(aliasConf).length) {
              return;
            } // reduce the configs to a single extensions array


            extensionsConf = conf.reduce((prev, curr) => {
              const next = [].concat(prev);
              const extensions = curr.extensions || curr.resolve && curr.resolve.extensions || [];

              if (extensions.length) {
                extensions.forEach(ext => {
                  if (next.indexOf(ext) === -1) {
                    next.push(ext);
                  }
                });
              }

              return next;
            }, []);

            if (!extensionsConf.length) {
              extensionsConf = null;
            }
          } else {
            // the exported webpack config is a single object...
            // use it's resolve.alias property
            aliasConf = conf.alias || conf.resolve && conf.resolve.alias; // use it's resolve.extensions property, if available

            extensionsConf = conf.extensions || conf.resolve && conf.resolve.extensions || [];
            if (!extensionsConf) extensionsConf = null;
          }

          aliases = aliasConf ? Object.keys(aliasConf) : [];
          cache.aliases = aliases;
          cache.aliasConf = aliasConf;
          cache.extensionsConf = extensionsConf;
        } // Get the path of the StringLiteral


        const _nodeArguments = _slicedToArray(nodeArguments, 1),
              filePath = _nodeArguments[0].value;

        for (const alias of aliases) {
          let aliasDestination = aliasConf[alias];
          const regex = new RegExp(`^${alias}(\/|$)`);

          if (regex.test(filePath)) {
            // notModuleRegExp from https://github.com/webpack/enhanced-resolve/blob/master/lib/Resolver.js
            const notModuleRegExp = /^\.$|^\.[\\\/]|^\.\.$|^\.\.[\/\\]|^\/|^[A-Z]:[\\\/]/i;
            const isModule = !notModuleRegExp.test(aliasDestination);

            if (isModule) {
              path.node.arguments = [_core.types.StringLiteral(aliasDestination)];
              return;
            } // If the filepath is not absolute, make it absolute


            if (!(0, _path.isAbsolute)(aliasDestination)) {
              aliasDestination = (0, _path.join)(cwd, aliasDestination);
            }

            let relativeFilePath = (0, _path.relative)((0, _path.dirname)(filename), aliasDestination); // In case the file path is the root of the alias, need to put a dot to avoid having an absolute path

            if (relativeFilePath.length === 0) relativeFilePath = '.';
            let requiredFilePath = filePath.replace(alias, relativeFilePath); // If the file is requiring the current directory which is the alias, add an extra slash

            if (requiredFilePath === '.') requiredFilePath = './'; // In the case of a file requiring a child directory of the current directory, we need to add a dot slash

            if (['.', '/'].indexOf(requiredFilePath[0]) === -1) {
              requiredFilePath = `./${requiredFilePath}`;
            } // TODO: should honor enforceExtension and then use extensionConf to make sure extension
            // In case the extension option is passed


            if (extensionsConf && !noOutputExtension) {
              // Get an absolute path to the file
              const absoluteRequire = (0, _path.join)(aliasDestination, (0, _path.basename)(filePath));
              let extension = null;
              (0, _lodash2.default)(extensionsConf, ext => {
                if (!ext) return false; // If the file with this extension exists set it

                if (fileExists(absoluteRequire + ext)) {
                  extension = ext;
                }

                return extension;
              }); // Set the extension to the file path, or keep the original one

              requiredFilePath += extension || '';
            }

            path.node.arguments = [_core.types.StringLiteral(requiredFilePath)];
            return;
          }
        }
      }

    }
  };
});

exports.default = _default;